
    const parser = require('./parser');
    const sourcemap = require('source-map');

    module.exports.fromFileContents = function fromFileContents(inputFilename, input) {


      const sourceMapGenerator = new sourcemap.SourceMapGenerator({
        file: inputFilename,
      });

      sourceMapGenerator.setSourceContent(inputFilename, input);


      const parsed = parser(input);

      const output = [];
      let outputLineNumber = 0;

      let indent = 0;
      let inCode = false;


      parsed.forEach(line => {


        if (inCode && typeof line.indent !== 'undefined' && line.indent <= indent) {
          inCode = false;
          indent = 0;
        }


        if (line.type === 'code') {
          inCode = true;
          indent = line.indent;
        }


        if (inCode && typeof line.text !== 'undefined') {
          outputLineNumber += 1;
          output.push(line.text);


          sourceMapGenerator.addMapping({
            source: inputFilename,
            original: {
              line: line.lineNumber,
              column: 0,
            },
            generated: {
              line: outputLineNumber,
              column: 0,
            },
          });
        }
      });


      output.push('\n');

      return {
        content: output.join('\n'),
        sourceMap: JSON.parse(sourceMapGenerator.toString()),
      };
    };


    const fs = require('fs');

    module.exports.fromFile = function fromFile(filename) {
      const content = fs.readFileSync(filename).toString();
      return exports.fromFileContents(filename, content);
    };

